/*
   libwrap.c  TCP Wrapper for IP v4/v6
   $Header: /home/fujiwara/.cvsroot/tcpd/rfc1413.c,v 1.11 2000/04/19 07:36:27 fujiwara Exp $
   tcp_wrappers_7.6 compatible package for IPv6+IPv4
   written by Kazunori Fujiwara <fujiwara@rcac.tdi.co.jp>
 */

#include <setjmp.h>
#include <unistd.h>
#include <signal.h>
#include <netdb.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdio.h>
#ifdef NO_ADDRINFO
#include "addrinfo.h"
#endif
#include "tcpd.h"

#define IDENT_PORT 113
#define	STRINGLEN  256

static int writes(int sock, char *buf)
{
	char *p;
	int i, len;

	p = buf;
	len = strlen(buf);

	while(len > 0) {
		if ((i = write(sock, p, len)) <= 0)
			return 0;
		len -= i;
		p += i;
	}
	return 1;
}

static int read1(int sock, char *buf, int len)
{
	char *p, *q;
	int i, left;

	p = buf;
	left = len - 1;
	while((i = read(sock, p, left)) > 0) {
		q = p;
		p += i;
		left -= i;
		*p = '\0';
		if (strchr(q, '\n') != NULL)
			break;
	}
	return i > 0;
}

static int timeout_happen;

static void timeout(int sig)
{
	timeout_happen = 1;
}

char *rfc1413(
	struct sockaddr *local,
	struct sockaddr *remote,
	 char *user, int userlen, int rfc1413_timeout)
{
	int s, rp, lp, salen;
	int nrp, nlp;
	char *p, *ret = NULL;
	struct sockaddr_storage sal, sar;
	char buf[STRINGLEN];
	char buf2[STRINGLEN];

	if (rfc1413_timeout <= 0)
		return NULL;

	user[0] = '\0';
	memcpy(&sar, remote, remote->sa_len);
	memcpy(&sal, local, local->sa_len);
	switch(remote->sa_family) {
	case AF_INET:
		((struct sockaddr_in *)&sar)->sin_port = htons(IDENT_PORT);
		((struct sockaddr_in *)&sal)->sin_port = 0;
		rp = ntohs(((struct sockaddr_in *)remote)->sin_port);
		lp = ntohs(((struct sockaddr_in *)local)->sin_port);
		salen = sizeof(struct sockaddr_in);
		break;
#ifdef INET6
	case AF_INET6:
		((struct sockaddr_in6 *)&sar)->sin6_port = htons(IDENT_PORT);
		((struct sockaddr_in6 *)&sal)->sin6_port = 0;
		rp = ntohs(((struct sockaddr_in6 *)remote)->sin6_port);
		lp = ntohs(((struct sockaddr_in6 *)local)->sin6_port);
		salen = sizeof(struct sockaddr_in6);
		break;
#endif
	default:
		return NULL;
	}
	snprintf(buf, sizeof(buf), "%u,%u\r\n", rp, lp);
	if ((s = socket(remote->sa_family, SOCK_STREAM, 0)) < 0)
		return NULL;
	timeout_happen = 0;
	signal(SIGALRM, timeout);
	alarm(rfc1413_timeout);
	if (bind(s, (struct sockaddr *)&sal, salen) == 0
	  && connect(s, (struct sockaddr *)&sar, salen) >= 0
	  && writes(s, buf)
	  && read1(s, buf, sizeof(buf))
	  && sscanf(buf, "%u , %u : USERID :%*[^:]:%255s",
		&nrp, &nlp, buf2) == 3
	  && nlp == lp && nrp == rp) {
		if ((p = strchr(buf2, '\r')) != NULL)
			*p = '\0';
		strncpy(user, buf2, userlen);
		ret = user;
	}
	alarm(0);
	close(s);
	signal(SIGALRM, SIG_IGN);

	return ret;
}

